C
C update with new parameter values from Grosse and
C a new determination of the beta_eff with valence nucleons
C also read the charge radius parameter from the 5dch table !
C calculate F(E1) tables for use with Talys 1.4 from the F(E1) parametrisation
C TRIPLE LORENTZIAN (TLO) PARAMETRIZATION
C Junghans et al. PLB 670 (2008) 200
C E. Grosse et al., Eur. Phys. J. A (2017) 53: 225
C
C beta gs deformation and triaxiality gamma taken from the table 5dch.txt from
C DelaRoche et al. PHYSICAL REVIEW C 81, 014303 (2010)
C
C odd isotopes are interpolated from their neighbors
C
      PROGRAM MAIN
      IMPLICIT NONE
      REAL*8 BETAGS(110,198),BETAGS_ERR(100,160),BETA,BETA_ERR
      REAL*8 GAMMAGS(110,198),F1PARA,DELTA_P,DELTA_N,DELTA
      REAL*8 BETA_FRDM(132,250),BETA_RAMAN(100,160),DBETA
      REAL*8 RC(110,198),RCH 
      REAL*8 HBARC, KE1,PI,BGS,GGS,TEST,ETEST,GTEST
C Variable names from 5dch.txt
      REAL*8 EHFB,BMIN,GMIN,RCHFB,RPHFB,RNHFB,EABS,ECORR
      REAL*8 BET01,GAM01,DELB01,DELG01,E21,E41,E61,E02,E22,
     &       E23,PK21,PK22,PK23,BE21,BE22,BE23,BE24,BE25,BE26,
     &       RC5DCH,RP5DCH,RN5DCH,ROE0TH      
C
      INTEGER IA,IZ,IN,INMIN,INMAX,IZMIN,IZMAX,IFLAG(110,198)
      INTEGER FLAG
      CHARACTER*2 ELEMENT
      CHARACTER*80 DUMMY
      CHARACTER*4 FILENAME
      CHARACTER*12 FNAME
      CHARACTER*3 FNUM
C
C hbar*c in MeV*fm
C
      PI = 4.D0 * DATAN(1.D0)
      HBARC = 197.3269788D0
C
C in units (MeV * fm) ** (-2)
C
      KE1 =  1.D0 / (3.D0 * (PI * HBARC)**2)
C      write(6,*) 'KE1',KE1
      DO IZ=1,110,1
        DO IN = 1,198,1
           BETAGS(IZ,IN) = 99.D0
           GAMMAGS(IZ,IN) = 99.D0
           IFLAG(IZ,IN) = 0
        END DO
      END DO
C
C READ BETAGS GAMMAGS from 5dch.txt Delaroche Table 
C
      OPEN(FILE='5dch.txt',UNIT=12,STATUS='old')
      READ(UNIT=12,FMT=*) DUMMY
 5    READ(UNIT=12,FMT=*,END=10) IZ,IN,EHFB,BMIN,GMIN,
     & RCHFB,RPHFB,RNHFB,EABS,ECORR,BET01,GAM01,DELB01,DELG01,
     & E21,E41,E61,E02,E22,E23,PK21,PK22,PK23,BE21,BE22,
     & BE23,BE24,BE25,BE26,RC5DCH,RP5DCH,RN5DCH,ROE0TH 
     
      WRITE(6,'(2I5,2F7.3)') IZ,IN,BET01,GAM01,RP5DCH 
C 
C equivalent charged sphere radius from 5DCH table
C
      RC(IZ,IN) = DSQRT(5.D0/3.D0) * RP5DCH  
C
C Reduce beta of nuclides delta <= 10 nucleons from the shell
C  to (0.40 + delta/20 ) of the original Delaroche Value
C 
C A.R.J. 01.09.2016
C   DELTA used in correction factor (100 means no correction) 
      DELTA_N = 100.D0
	  DELTA_P = 100.D0
C
C	 magic number 28  
      IF ((IZ.GE.18).AND.(IZ.LE.38)) THEN 
	    DELTA_P = ABS(28-IZ)
      END IF
	  IF ((IN.GE.18).AND.(IN.LE.38)) THEN 
        DELTA_N = ABS(28-IN)
      END IF
C    magic number 50
      IF ((IZ.GE.40).AND.(IZ.LE.60)) THEN 
	    DELTA_P = ABS(50-IZ)
      END IF
	  IF ((IN.GE.40).AND.(IN.LE.60)) THEN 
        DELTA_N = ABS(50-IN)
      END IF
C    magic number 82
      IF ((IZ.GE.72).AND.(IZ.LE.92)) THEN 
	    DELTA_P = ABS(82-IZ)
      END IF
	  IF ((IN.GE.72).AND.(IN.LE.92)) THEN 
        DELTA_N = ABS(82-IN)
      END IF 
C    magic number 126
      IF ((IZ.GE.116).AND.(IZ.LE.136)) THEN 
	    DELTA_P = ABS(126-IZ)
      END IF
	  IF ((IN.GE.116).AND.(IN.LE.136)) THEN 
        DELTA_N = ABS(126-IN)
      END IF 
C DELTA is either 100 meaning DELTA_N or DELTA_P are more than 10 nucleons 
C away from the shell. Or it is the maximum of both if they are less than 10 nucleons away 
C 
       IF ((DELTA_N.LE.10.D0).AND.(DELTA_P.LE.10.D0)) THEN
 	      DELTA = MAX(DELTA_N,DELTA_P)
       ELSE 
           DELTA = MIN(DELTA_N,DELTA_P)
       END IF 
C 
	  IF (DELTA.LE.10.D0) THEN
	     BETAGS(IZ,IN) = BET01 * (0.4D0 + DELTA/20.D0)	
C
         WRITE(6,FMT='(A10,2I5,2F6.3)') 'corrected ',IZ,IN,
     &    BETAGS(IZ,IN),BET01
         IFLAG(IZ,IN) = 1 
C	 
	  ELSE  
	      BETAGS(IZ,IN) = BET01
		  WRITE(6,FMT='(A10,2I5,2F6.3)') 
     &    'no corr.  ',IZ,IN,BETAGS(IZ,IN)
	  END IF
C
      GAMMAGS(IZ,IN) = GAM01
C
C If no deformation data in input file set value to 99. to flag the data later
C
	  IF (BET01.GE.98.D0)  BETAGS(IZ,IN) = BET01
C      IF (BETAGS(IZ,IN).GT.0.6) Write(6,*) 'error in betags ',IZ,IN  
C Catch last line before missing data in 5dch.txt
C replaced question marks with 99. in the input file 5dch.txt
C
C      IF ((IZ.eq.98).and.(IN.eq.128)) GO TO 10	   
      GO TO 5
  10  CONTINUE  
      WRITE(6,*) '5DCH table read in'
      CLOSE(UNIT=12)
C 
C  INTERPOLATE THE ODD N ISOTOPES
C  
      DO IZ = 10,110,2
        DO IN =2,196,2
          IF (BETAGS(IZ,IN).LT.99.D0.AND.BETAGS(IZ,IN+2).LT.99.D0) THEN
C          
            RC(IZ,IN+1) = (RC(IZ,IN) + RC(IZ,IN+2))/2.D0
C
            BETAGS(IZ,IN+1) = (BETAGS(IZ,IN) + BETAGS(IZ,IN+2))/2.D0
            GAMMAGS(IZ,IN+1) = (GAMMAGS(IZ,IN) + GAMMAGS(IZ,IN+2))/2.D0
        IF((GAMMAGS(IZ,IN+1).GT.60.D0).OR.(BETAGS(IZ,IN+1).GT.1.D0))THEN
             WRITE(6,*) 'ERROR IN ODD N ISOTOPE INTERPOLATION',IZ,IN
			 write(6,*) BETAGS(IZ,IN),BETAGS(IZ,IN+2)
             STOP
            END IF
            IFLAG(IZ,IN+1) = 2
          END IF
        END DO
      END DO
C
C INTERPOLATE THE ODD Z NUCLIDES
C
      DO IZ = 11,109,2
        DO IN =1,198,1
        IF (BETAGS(IZ-1,IN).LT.99.D0.AND.BETAGS(IZ+1,IN).LT.99.D0) THEN
C          
            RC(IZ,IN) = (RC(IZ-1,IN) + RC(IZ+1,IN))/2.D0
C        
         BETAGS(IZ,IN) =(BETAGS(IZ-1,IN) + BETAGS(IZ+1,IN))/2.D0
         GAMMAGS(IZ,IN) =(GAMMAGS(IZ-1,IN) + GAMMAGS(IZ+1,IN))/2.D0
        IF((GAMMAGS(IZ,IN).GT.60.D0).OR.(BETAGS(IZ,IN).GT.1.D0))THEN
             WRITE(6,*) 'ERROR IN ODD Z NUCLIDE INTERPOLATION',IZ,IN
             STOP
        END IF
        IFLAG(IZ,IN) = 2
        END IF
        END DO
      END DO
C
C loop over the elements: START WITH SELENIUM 
C
      IZMIN = 34
C      IZMIN = 92
      IZMAX = 110
      do IZ = IZMIN,IZMAX,1
C
C loop over the isotopes of one element
C 

       IF (IZ.LT.100) THEN 
         WRITE(FNUM,40) IZ
 40      format(I2)
         FILENAME = 'z0'//FNUM
       ELSE
         WRITE(FNUM,44) IZ
 44      format(I3)
         FILENAME = 'z'//FNUM
       ENDIF
       write(6,*) FILENAME
C
C open output file
C
       OPEN(UNIT=15,FILE=FILENAME)
       FNAME = 'FE1_'//FILENAME//'.dat'
       OPEN(UNIT=16,FILE=FNAME)
C
       do IN = 1,198,1
        IA = IN+IZ
        RCH = RC(IZ,IN)
        BGS = BETAGS(IZ,IN)
        GGS = GAMMAGS(IZ,IN)
        FLAG = IFLAG(IZ,IN)
        IF ((BGS.LT.98.D0).AND.(GGS.LT.98.D0)) THEN
           CALL WRITEISOTOPE(IZ,IN,RCH,BGS,GGS,FLAG)
        END IF
       END DO
       CLOSE(UNIT=15)
      end do
C
C end of main
C 
      END

C
       SUBROUTINE WRITEISOTOPE(IZ,IN,RCH,BGS,GGS,FLAG)
C
       IMPLICIT NONE
       INTEGER IN,IZ,J,IA,FLAG
       REAL*8 PI,HBARC
       REAL*8 EGAM ,F1PARA,BGS,GGS,FE1,FE1_MB,KE1,DUM
       REAL*8 EX,EY,EZ,R,P,Q,RCH
       CHARACTER*5 CFLAG
       external F1PARA
C
C hbar*c in MeV*fm
C
       DUM = 0.D0
       PI = 4.D0 * DATAN(1.D0)
      HBARC = 197.3269788D0
C
C in units (MeV * fm) ** (-2)
C
       KE1 =  1.D0 / (3.D0 * (PI * HBARC)**2)
C       write(6,*) 'KE1 ',KE1
       IA = IN + IZ
       IF (FLAG.eq.0) CFLAG = ' 5DCH'
       IF (FLAG.eq.1) CFLAG = '  EFF'
       IF (FLAG.eq.2) CFLAG = 'INTER'
       write(6,'(A4,I3,A4,I3,A5,F6.3,A5,F7.3,1x,A5)') ' Z= ',IZ,
     & ' A= ',IA,' bet=',BGS,' gam=',GGS,CFLAG
C test output for parameter list    
       write(12,'(A4,I3,A4,I3,A5,F7.3,A5,F6.3,A6,F7.3,1x,A5)') 
     &  ' Z= ',IZ,' A= ',IA,' RC= ',RCH,' bet=',BGS,' gam= ',GGS,CFLAG
C       write(6,*) ' U[MeV]  fE1[mb/MeV]'
       write(15,'(A4,I3,A4,I3,A5,F6.3,A5,F7.3,1x,A5)') ' Z= ',IZ,
     & ' A= ',IA,' bet=',BGS,' gam=',GGS,CFLAG
       write(15,*) ' U[MeV]  fE1[mb/MeV]'
       write(16,'(A4,I3,A4,I3,A5,F6.3,A5,F7.3,1x,A5)') ' Z= ',IZ,
     & ' A= ',IA,' bet=',BGS,' gam=',GGS,CFLAG
       write(16,*) ' U[MeV]  fE1[MeV**-3]'
       EGAM =  0.D0
       DO J= 1,300,1
        EGAM = 0.100D0 + EGAM
        FE1 =  F1PARA(IZ,IN,RCH,BGS,GGS,EGAM,EX,EY,EZ,R,P,Q)
C
C convert from MeV**-3 to units of mbarn/MeV
C
        FE1_MB = FE1 / KE1 * 10.D0
C
C        write(6,30) EGAM,FE1_MB
        write(15,30) EGAM,FE1_MB
        write(16,32) EGAM,DUM,FE1,DUM
C        write(6,*) 'FE1:',EGAM,FE1
  30    FORMAT(F9.3,2x,ES10.3)
  32    FORMAT(2F10.3,2E12.3)
       END DO
C 
C OUTPUT 
C    
       WRITE(6,*) 'EX,EY,EZ',EX,EY,EZ
       WRITE(6,*) 'W1,W2,W3',R,(R/P),(R/Q)
       write(6,*) '    '
       write(15,*)'    '
       END
C
C
C F(E1) parametrization junghans et al. phys. lett b 670 (2008) 200
C calc. and parameters updated as published E. Grosse et al., Eur. Phys. J. A (2017) 53: 225
C F(E1) in units of 1/MeV**3
C sigma_abs = 3 * (pi * hbarc)**2 * Egam * F(E1)
C convert F(E1) to units of mb/MeV by  multiplication with
C  3 * (pi * hbarc)**2 * 10 (hbarc given in MeV fm)
C
C E1 dipole strength at Energy EGAM as function of
C IZ, IN, Neutron and Proton number of nucleus
C beta, gamma gs deformation and triaxiality,
C gamma ray energy
C
      REAL*8 FUNCTION F1PARA(IZ,IN,RCH,BETA_GS,GAMMA_GS,EGAM,EX,
     & EY,EZ,R,P,Q)
      IMPLICIT NONE
      INTEGER IZ,IN,IA,K
      REAL*8 BETA_GS,GAMMA_GS,EGAM,SIG_ABS,GAM_RAD
      REAL*8 TRK(3), ERES(3), E0,LOR(3),GAM(3),RE(3)
      REAL*8 U,EPS,J,Q1,MSTAR,R0,HBARC,C1,PI
      REAL*8 EX,EY,EZ,R,P,Q,X,RCH
C
C FRDLM parameters
C
      SIG_ABS = 0.D0
      F1PARA = 0.D0
C
C hbar*c in MeV*fm
C
      HBARC = 197.3269788D0
      PI = 4.D0 * DATAN(1.d0)
      C1 = DSQRT(5.D0/(4.D0 * PI))
      IA = IN + IZ
      EPS = 0.0768D0
      J = 32.7D0
      Q1 = 29.2D0
C
      U = (1.D0 - EPS) * DBLE(IA)**(-1.D0/3.D0) * 3.D0 * J / Q1
C
C  800 MeV effective mass arxive preprint 06.09.2016
C   used only with new radius parameter from 5DCH.txt
C  this is an input parameter to the function (RCH)
C
      MSTAR = 800.D0
C
C     R0 = 1.16D0*DBLE(IA)**(1.D0/3.D0)
C        
      R0 = RCH
C
C centroid energy
C
      E0 = HBARC/R0 * DSQRT( 8.D0*J/MSTAR * DBLE(IA)**2/
     & (4.D0*DBLE(IN*IZ))) * ( 1.D0 + U  - EPS *
     & (1.D0 + EPS + 3.D0 * U)/(1 + EPS + U))**(-0.5D0)
C
      GAM_RAD = GAMMA_GS * PI/180.D0
C
C      write(6,*) 'U ',U
C      write(6,*) 'E0 ',E0
C      write(6,*) 'EGAM', EGAM
C
C
C Ratios of triaxial oscillator frequencies Delaroche et al. Eq.(3)
C Calculated backwards from the given beta, gamma values.
C      
      X = BETA_GS/(2.D0*BETA_GS + 1.D0)
      P = EXP(-X*DSQRT(3.D0)*DSIN(GAM_RAD))
      Q = EXP(X*(1.5D0 * DCOS(GAM_RAD)-DSQRT(3.D0)/2.D0 * 
     &    DSIN(GAM_RAD)))
      R = (P * Q)**(1.D0/3.D0)
      RE(1) = R
      RE(2) = R/P
      RE(3) = R/Q
C
      DO K = 1,3
        TRK(K) =5.97D0*2.D0/PI/3.D0 * DBLE(IZ*IN)/DBLE(IA)
C        ERES(K) = E0 /DEXP(C1*BETA_GS*DCOS(GAM_RAD-2.D0/3.D0*K*PI))
        ERES(K) = E0 * RE(K)
C
C        GAM(K) = 0.05D0 * ERES(K)**1.6D0
C
C to improve GDR peak region A.R.J. 09.07.2012
C
        GAM(K) = 0.045D0 * ERES(K)**1.6D0 
C        
        LOR(K) = EGAM**2 * GAM(K) / ((ERES(K)**2 -EGAM**2)**2 +
     &  EGAM**2 * GAM(K)**2 )
        SIG_ABS = SIG_ABS + TRK(K) * LOR(K)
C        write(6,*) K,TRK(K),ERES(K),GAM(K),(TRK(K)*LOR(K))
      END DO
C
C Ratio of centroid energies
C      
      EX = ERES(1)/E0
      EY = ERES(2)/E0
      EZ = ERES(3)/E0   
C
C SIG_ABS in fm2  convert to mbarn
C
        SIG_ABS = SIG_ABS*10.D0
C        write(6,*) 'sigma_abs ',SIG_ABS
C
C units of F1PARA are fm2 / MeV / MeV**2 fm2  = 1 / MeV**3
C
        F1PARA = SIG_ABS / EGAM / 3.D0 /10.D0 / (PI * HBARC)**2
C
C end of function F1PARA
C
      END
